package im.getsocial;

import androidx.annotation.NonNull;

import im.getsocial.sdk.Callback;
import im.getsocial.sdk.json.GetSocialJson;
import im.getsocial.sdk.GetSocial;
import im.getsocial.sdk.communities.CurrentUser;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** GetSocialPlugin */
public class GetSocialPlugin implements FlutterPlugin, MethodCallHandler, StreamHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel methodsChannel;
  private EventChannel onInitializeChannel;
  private EventChannel onCurrentUserChangedChannel;
  private EventChannel onReferralDataReceivedChannel;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    methodsChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "im.getsocial.flutter/methods");
    methodsChannel.setMethodCallHandler(this);
    onInitializeChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), "im.getsocial.flutter/event/initialization");
    onInitializeChannel.setStreamHandler(this);
    onCurrentUserChangedChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), "im.getsocial.flutter/event/currentuserchanged");
    onCurrentUserChangedChannel.setStreamHandler(this);
    onReferralDataReceivedChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(), "im.getsocial.flutter/event/referraldatareceived");
    onReferralDataReceivedChannel.setStreamHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel methodsChannel = new MethodChannel(registrar.messenger(), "im.getsocial.flutter/methods");
    methodsChannel.setMethodCallHandler(new GetSocialPlugin());
    final EventChannel onInitializeChannel = new EventChannel(registrar.messenger(), "im.getsocial.flutter/event/initialization");
    onInitializeChannel.setStreamHandler(new GetSocialPlugin());
    final EventChannel onCurrentUserChangedChannel = new EventChannel(registrar.messenger(), "im.getsocial.flutter/event/currentuserchanged");
    onCurrentUserChangedChannel.setStreamHandler(new GetSocialPlugin());
    final EventChannel onReferralDataReceivedChannel = new EventChannel(registrar.messenger(), "im.getsocial.flutter/event/referraldatareceived");
    onReferralDataReceivedChannel.setStreamHandler(new GetSocialPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, final @NonNull Result result) {
    String name = call.method;
    String method = call.hasArgument("method") ? call.<String>argument("method") : null;
    String body = call.hasArgument("body") ? call.<String>argument("body") : null;
    if (body != null) {
      body = body.replaceAll("GETSOCIAL_CURRENT_USER_PLACEHOLDER_42_42", GetSocial.getCurrentUser().getId());
    }
    switch (name) {
      case "callSync":
        result.success(GetSocialJson.callSync(method, body));
        break;
      case "callAsync":
        GetSocialJson.callAsync(method, body, new Callback<String>() {
          @Override
          public void onSuccess(String s) {
            result.success(s);
          }
        }, new Callback<String>() {
          @Override
          public void onSuccess(String s) {
            result.error("0", s, null);
          }
        });
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    methodsChannel.setMethodCallHandler(null);
  }

  @Override
  public void onListen(Object arguments, final EventSink events) {
    if (arguments instanceof String) {
      final String listener = (String) arguments;
      GetSocialJson.registerListener(listener, new Callback<String>() {
        @Override
        public void onSuccess(String s) {
          events.success(s);
        }
      });
    }
  }

  @Override
  public void onCancel(Object arguments) {
  }
}
