import Flutter
import UIKit
import GetSocialSDK

public class SwiftGetSocialPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "im.getsocial.flutter/methods", binaryMessenger: registrar.messenger())
    channel.setMethodCallHandler { (methodCall, result) in
        let name = methodCall.method
        var method: String = ""
        var body: String = ""

        if let arguments = methodCall.arguments as? NSDictionary {
            method = arguments["method"] as? String ?? ""
            body = arguments["body"] as? String ?? ""
        }
        body = body.replacingOccurrences(of: "GETSOCIAL_CURRENT_USER_PLACEHOLDER_42_42", with: GetSocial.currentUser()?.userId ?? "")
        
        if method == "Communities.removeActivities" {
            body = body.replacingOccurrences(of: "ids", with: "activityIds")
        }
        switch (name) {
            case "callSync":
                result(JSONBridge.callSync(method: method, parameter: body))
            case "callAsync":
                JSONBridge.callAsync(method: method, parameter: body, success: { pluginResult in
                    result(pluginResult)
                }, failure: { error in
                    result(FlutterError(code: "0", message: error, details: nil))
                })
            default:
                result(FlutterError(code: "0", message: "Unknown method \(name)", details: nil))
        }
    }

    let onInitializeChannel = FlutterEventChannel(name: "im.getsocial.flutter/event/initialization", binaryMessenger: registrar.messenger())
    onInitializeChannel.setStreamHandler(SwiftGetSocialPlugin());
    let onCurrentUserChangedChannel = FlutterEventChannel(name: "im.getsocial.flutter/event/currentuserchanged", binaryMessenger: registrar.messenger())
    onCurrentUserChangedChannel.setStreamHandler(SwiftGetSocialPlugin());
    let onReferralDataReceivedChannel = FlutterEventChannel(name: "im.getsocial.flutter/event/referraldatareceived", binaryMessenger: registrar.messenger())
    onReferralDataReceivedChannel.setStreamHandler(SwiftGetSocialPlugin());
  }

}

extension SwiftGetSocialPlugin: FlutterStreamHandler {
    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        if let method = arguments as? String {
            _ = JSONBridge.addListener(method: method, success: { result in
                events(result)
            })
        }
        return nil
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        return nil
    }
}
