import 'codable.dart';

/// Describes MediaAttachment class.
class MediaAttachment extends Encodable {
  String imageUrl;
  String base64Image;
  String videoUrl;
  String base64Video;
  String gifUrl;

  MediaAttachment();

  MediaAttachment.fromJSON(Map<String, dynamic> json)
      : imageUrl = json['imageUrl'],
        videoUrl = json['videoUrl'],
        gifUrl = json['gifUrl'];

  ///
  /// Create an attachment with image URL.
  /// [imageUrl] image URL.
  ///
  static withImageUrl(String imageUrl) {
    MediaAttachment m = new MediaAttachment();
    m.imageUrl = imageUrl;
    return m;
  }

  ///
  /// Create an attachment with Base64 encoded image.
  ///
  /// [base64Image] image URL.
  ///
  static withBase64Image(String base64Image) {
    MediaAttachment m = new MediaAttachment();
    m.base64Image = base64Image;
    return m;
  }

  ///
  /// Create an attachment with video URL.
  ///
  /// [videoUrl] video URL.
  ///
  static withVideoUrl(String videoUrl) {
    MediaAttachment m = new MediaAttachment();
    m.videoUrl = videoUrl;
    return m;
  }

  ///
  /// Create an attachment with Base64 encoded video.
  ///
  /// [base64Video] video URL.
  ///
  static withBase64Video(String base64Video) {
    MediaAttachment m = new MediaAttachment();
    m.base64Video = base64Video;
    return m;
  }

  Map<String, dynamic> toJSON() => {
        'imageUrl': imageUrl,
        'videoUrl': videoUrl,
        'image': base64Image,
        'video': base64Video
      };

  @override
  String toString() {
    return 'MediaAttachment{'
        'imageUrl: $imageUrl, '
        'videoUrl: $videoUrl, '
        'gifUrl: $gifUrl, '
        '}';
  }
}
