import 'codable.dart';

class UserId extends Encodable {
  String _id;
  String _providerId;

  ///
  /// Create a single GetSocial user identifier.
  ///
  /// [id]    GetSocial user ID.
  ///
  static UserId create(String id) {
    UserId instance = new UserId();
    instance._id = id;
    return instance;
  }

  ///
  /// Create a single identity user identifier for giver provider ID.
  ///
  /// [id]          Custom identity provider ID.
  /// [providerId]  Identity user ID.
  ///
  static UserId createWithProvider(String id, String providerId) {
    UserId instance = new UserId();
    instance._id = id;
    instance._providerId = providerId;
    return instance;
  }

  ///
  /// Return UserId of current user.
  ///
  static UserId currentUser() {
    UserId instance = new UserId();
    instance._id = 'GETSOCIAL_CURRENT_USER_PLACEHOLDER_42_42';
    return instance;
  }

  Map<String, dynamic> toJSON() => {
        'userId': _id,
        'provider': (_providerId == null || _providerId.length == 0)
            ? null
            : _providerId
      };

  String toString() {
    if (_providerId == null) {
      return _id;
    }
    return '$_providerId:$_id';
  }
}
