import '../common/media_attachment.dart';
import '../src/utils.dart';
import '../usermanagement/user.dart';
import 'activitybutton.dart';
import 'communitiesentity.dart';
import 'mention.dart';
import 'userreactions.dart';

/// Describe Activity object.
class Activity {
  Activity.fromJSON(Map<String, dynamic> json)
      : id = json['id'],
        text = json['text'],
        author = User.fromJson(json['author']),
        attachments = Utils.createList(json['attachments'],
            (jsonMap) => MediaAttachment.fromJSON(jsonMap)),
        button = json['button'] == null
            ? null
            : ActivityButton.fromJSON(json['button']),
        type = json['type'],
        isAnnouncement = json['announcement'],
        commentsCount = json['commentsCount'],
        reactionsCount = Utils.createStringIntMap(json['reactionsCount']),
        myReactions = Utils.createStringList(json['myReactions']),
        reactions = Utils.createList(
            json['reactions'], (jsonMap) => UserReactions.fromJSON(jsonMap)),
        commenters = Utils.createList(
            json['commenters'], (jsonMap) => User.fromJson(jsonMap)),
        properties = Utils.createStringStringMap(json['properties']),
        createdAt = json['createdAt'],
        mentions = Utils.createList(
            json['mentions'], (jsonMap) => Mention.fromJSON(jsonMap)),
        viewCount = json['viewCount'],
        source = CommunitiesEntity.fromJSON(json['source']),
        status = json['status'];

  String id;
  String text;
  User author;
  List<MediaAttachment> attachments;
  ActivityButton button;
  String type;
  bool isAnnouncement;
  int commentsCount;
  Map<String, int> reactionsCount;
  List<String> myReactions;
  List<UserReactions> reactions;
  List<User> commenters;
  Map<String, String> properties;
  int createdAt;
  List<Mention> mentions;
  int viewCount;
  CommunitiesEntity source;
  String status;

  @override
  String toString() {
    var authorId = this.author.userId;
    return 'Activity{'
        'id: $id, '
        'text: $text,'
        'author: $authorId,'
        'mediaAttachments: $attachments,'
        'button: $button,'
        'type: $type,'
        'isAnnouncement: $isAnnouncement,'
        'commentsCount: $commentsCount,'
        'reactionsCount: $reactionsCount,'
        'myReactions: $myReactions,'
        'reactions: $reactions,'
        'commenters: $commenters,'
        'properties: $properties,'
        'createdAt: $createdAt,'
        'mentions: $mentions,'
        'viewCount: $viewCount,'
        'source: $source,'
        'status: $status,'
        '}';
  }
}
