import '../../common/codable.dart';
import '../../common/userid.dart';
import '../../src/communitiesids.dart';
import '../communitiesentity.dart';
import 'announcementsquery.dart';

class ActivitiesQuery extends Encodable {
  CommunitiesIds ids;
  UserId author;
  String tag;

  ActivitiesQuery(CommunitiesIds ids) {
    this.ids = ids;
  }

  static ActivitiesQuery feedOf(UserId id) {
    return ActivitiesQuery(CommunitiesIds.user(id));
  }

  static ActivitiesQuery inTopic(String id) {
    return ActivitiesQuery(CommunitiesIds.topic(id));
  }

  static ActivitiesQuery inAllTopics() {
    return ActivitiesQuery(CommunitiesIds.topics([]));
  }

  static ActivitiesQuery timeline() {
    return ActivitiesQuery(CommunitiesIds.timeline());
  }

  static ActivitiesQuery everywhere() {
    return ActivitiesQuery(CommunitiesIds.everywhere());
  }

  static ActivitiesQuery commentsToActivity(String id) {
    return ActivitiesQuery(CommunitiesIds.activity(id));
  }

  ActivitiesQuery byUser(UserId id) {
    this.author = id;
    return this;
  }

  ActivitiesQuery withTag(String tag) {
    this.tag = tag;
    return this;
  }

  AnnouncementsQuery asAnnouncementsQuery() {
    switch (this.ids.type) {
      case CommunitiesEntityType.topic:
        return AnnouncementsQuery.inTopic(this.ids.ids.first);
      case CommunitiesEntityType.app:
        return AnnouncementsQuery.timeline();
      case CommunitiesEntityType.user:
        return AnnouncementsQuery.forFeedOf(UserId.create(this.ids.ids.first));
    }
    return null;
  }

  Map<String, dynamic> toJSON() => {
        'ids': ids.toJSON(),
        'author': author == null ? null : author.toJSON(),
        'tag': tag
      };
}
