import '../common/codable.dart';
import '../common/media_attachment.dart';

/// Available link parameter keys.
class LinkParamsKeys {
  /// Custom landing page title.
  static const String customTitle = '\$title';

  /// Custom landing page description.
  static const String customDescription = '\$description';

  /// Custom landing page image.
  static const String customImage = '\$image';

  /// Custom landing page YouTube video.
  static const String customYouTubeVideo = '\$youtube_video';

  /// Promo code attached to the invite.
  static const String promoCode = '\$promo_code';
}

/// Available invite content placeholders, they will be replaced with exact values.
class InviteContentPlaceholders {
  /// Invite link
  static const String appInviteUrl = "[APP_INVITE_URL]";

  /// Current user's display name.
  static const String userName = "[USER_NAME]";

  /// Promo code.
  static const String promoCode = "[PROMO_CODE]";
}

/// Describe InviteContent object.
class InviteContent extends Encodable {
  String subject;
  String text;
  MediaAttachment mediaAttachment;
  Map<String, String> linkParams;

  Map<String, dynamic> toJSON() => {
        'subject': subject,
        'text': text,
        'mediaAttachment':
            mediaAttachment == null ? null : mediaAttachment.toJSON(),
        'linkParams': linkParams == null ? null : linkParams
      };

  @override
  String toString() {
    return 'InviteContent{ '
        'subject: $subject,'
        'text: $text,'
        'mediaAttachment: $mediaAttachment,'
        'linkParams: $linkParams'
        '}';
  }
}
