import 'package:flutter/services.dart';

class NativeBridge {
  static const MethodChannel _methods =
      const MethodChannel('im.getsocial.flutter/methods');
  static const EventChannel _initializationEvent =
      const EventChannel("im.getsocial.flutter/event/initialization");
  static const EventChannel _currentUserChangedEvent =
      const EventChannel("im.getsocial.flutter/event/currentuserchanged");
  static const EventChannel _referralDataReceivedEvent =
      const EventChannel("im.getsocial.flutter/event/referraldatareceived");

  static Future<String> sync(String method, String body) async {
    final String response = await _methods
        .invokeMethod('callSync', {'body': body, 'method': method});
    return response;
  }

  static Future<String> async(String method, String body) async {
    final String response = await _methods
        .invokeMethod('callAsync', {'body': body, 'method': method});
    return response;
  }

  static void registerListener(String name, Function(String) listener) {
    if (name == 'GetSocial.addOnInitializedListener') {
      _initializationEvent.receiveBroadcastStream(name).listen((any) => {
            print('INVOKED: GetSocial.addOnInitializedListener'),
            listener(any as String)
          });
    } else if (name == 'GetSocial.addOnCurrentUserChangedListener') {
      _currentUserChangedEvent.receiveBroadcastStream(name).listen((any) => {
            print('INVOKED: GetSocial.addOnCurrentUserChangedListener'),
            listener(any as String)
          });
    } else if (name == 'Invites.setOnReferralDataReceivedListener') {
      _referralDataReceivedEvent.receiveBroadcastStream(name).listen((any) => {
            print('INVOKED: Invites.setOnReferralDataReceivedListener'),
            listener(any as String)
          });
    }
  }
}
