import '../common/codable.dart';

class IdentityProviderIds {
  static const String facebook = "facebook";
}

class Identity extends Encodable {
  String _provider;
  String _userId;
  String _token;
  Identity._(String provider, String userId, String token)
      : _provider = provider,
        _userId = userId,
        _token = token;

  ///
  /// Create custom identity.
  ///
  /// [provider] Your custom provider name.
  /// [userId]             Unique user identifier for your custom provider.
  /// [token]        Password of the user for your custom provider.
  ///                           It's a string, provided by the developer and it will be
  ///                           required by the GetSocial SDK to validate any future
  ///                           intent to add this same identity to another user.
  ///
  Identity.custom(String provider, String userId, String token)
      : this._(provider, userId, token);

  ///
  /// Creates a Facebook identity with specified access token.
  ///
  /// [token] Token of Facebook user returned from FB SDK.
  ///
  Identity.facebook(String token)
      : this._(IdentityProviderIds.facebook, null, token);

  Map<String, dynamic> toJSON() => {
        'provider': _provider,
        'userId': _userId,
        'accessToken': _token,
      };
}
