class UserUpdate {
  String _displayName;
  String _avatarUrl;

  String _avatar64;

  Map<String, String> _publicProperties = {};
  Map<String, String> _privateProperties = {};

  ///
  /// Set the display name.
  ///
  /// [name] new display name.
  ///
  UserUpdate updateDisplayName(String name) {
    _displayName = name;
    return this;
  }

  ///
  /// Set the avatar URL.
  ///
  /// [avatarUrl]   New avatar URL.
  ///
  UserUpdate updateAvatarUrl(String avatarUrl) {
    _avatarUrl = avatarUrl;
    return this;
  }

  ///
  /// Set the avatar using a Base64 encoded image.
  ///
  /// [base64Avatar] New avatar URL.
  ///
  UserUpdate updateBase64Avatar(String base64Avatar) {
    _avatar64 = base64Avatar;
    return this;
  }

  UserUpdate setPublicProperty(String key, String value) {
    _publicProperties[key] = value;
    return this;
  }

  UserUpdate setPrivateProperty(String key, String value) {
    _privateProperties[key] = value;
    return this;
  }

  UserUpdate removePublicProperty(String key) {
    _publicProperties[key] = "";
    return this;
  }

  UserUpdate removePrivateProperty(String key) {
    _privateProperties[key] = "";
    return this;
  }

  Map<String, dynamic> toJson() => {
        'displayName': _displayName,
        'avatarUrl': _avatarUrl,
        'avatar': _avatar64,
        'publicProperties': _publicProperties,
        'privateProperties': _privateProperties,
      };
}
